library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

%IF_HAS_REAL%-- These lines are included only if there are real variables in the model.
%IF_HAS_REAL%-- In that case, an additional floating-point package must be included.
%IF_HAS_REAL%-- The package can be downloaded for free from http://www.eda.org/fphdl/.
%IF_HAS_REAL%library ieee_proposed;
%IF_HAS_REAL%use ieee_proposed.float_pkg.all;

-- Entity declaration
-- The inputs and outputs of the entity are exactly the inputs and output of the
-- top-level node of the model, and they have the same names.
entity RTflowCode is
   port(
      %BOOL_INPUT_NAMES% : in std_logic;
      %INT_INPUT_NAMES% : in integer;
      %REAL_INPUT_NAMES% : in float32;
      %BOOL_OUTPUT_NAMES% : out std_logic;
      %INT_OUTPUT_NAMES% : out integer;
      %REAL_OUTPUT_NAMES% : out float32;
      clk : in std_logic
   );
end RTflowCode;

architecture RTflowCode of RTflowCode is
   -- All the components
   %COMPONENTS%

   -- All the local signals
   signal %BOOL_SIGNALS% : std_logic;
   signal %INT_SIGNALS% : integer;
   signal %REAL_SIGNALS% : float32;

   -- Reset array - see comments for the reset array process below
   signal reset_array : std_logic_vector(3 downto 0) := (others => '0');
begin

   -- Reset array process
   -- The purpose of this process is to avoid race conditions during initalization of
   -- the chip. This is accomplished by keeping all signals in their intial state for
   -- the first four clock cycles. This process produces a four-bit array whose
   -- contents will be "0000", "1000", "1100", "1110" for the first four clock cycles,
   -- and from the fifth cycle and on, it will be "1111". In the state update process
   -- below, all signals are reset whenever this array is not "1111".
   process (clk)
   begin
      if clk'event and clk = '1' then
         reset_array <= '1' & reset_array(3 downto 1);
      end if;
   end process;

   -- Input synchronization and mapping process
   -- This process copies the inputs declared in the entity to the corresponding
   -- local signals, where they will be manipulated by the state update process.
   -- At the same time, the inputs are synchronized in order to minimize the risk
   -- for race conditions.
   process (clk)
   begin
      if clk'event and clk = '1' then
         %INPUT_MAP%
      end if;
   end process;

   -- State update process
   -- This process generates a set of clocked registers, corresponding to all Pre
   -- blocks in the model. The first four cycles, the registers are forced to
   -- their initial values.
   process (clk)
   begin
      if clk'event and clk = '1' then
         if reset_array = "1111" then
            -- Update state
            %STATE_UPDATE_EQUATIONS%
         else
            -- Reset state
            %INITIAL_EQUATIONS%
         end if;
      end if;
   end process;

   -- Combinatorial part
   -- These statements constitute the complete combinatorial network, corresponding
   -- to all blocks except for Pre blocks in the model.
   %OUTPUT_UPDATE_EQUATIONS%

   -- Output map
   -- These statements copy the produced output values in the local signals to
   -- the outputs declared in the entity.
   %OUTPUT_MAP%

end RTflowCode;

